// default list of locations
var locationsList = [
	{name:"Nobeyama", longitude:138.48},
	{name:"Chajnantor", longitude:-67.75},
	{name:"Mauna Kea", longitude:-155.47},
	{name:"CARMA", longitude:-118.142},
	{name:"Greenwich", longitude:0.0}
]

// timer variable
var timer = null;
var timerPlaying = 1;

// ID to set preference for each widget
var locationKey = widget.identifier + "-location"

// set event handlers for the widget
if (window.widget) {
	widget.onshow = startTimer;
	widget.onhide = stopTimer;
	widget.onremove = erasePrefs;
}

//--------------------------------
// functions
//--------------------------------

// FUNCTION startTimer
// initialize and start timer
//--------------------------------
function startTimer() {
	if (window.widget) {
		// read location list from preference file
		for(i=0; i<5; i++) {
			var location = widget.preferenceForKey("locListName"+i);
			if (location != null) locationsList[i].name = location;
			var longitude = widget.preferenceForKey("locListLong"+i);
			if (longitude != null) locationsList[i].longitude = longitude;
			updateLocationSelect();
		}
		var location = widget.preferenceForKey(locationKey);
		if (location != null) document.getElementById("location").value = location;
	}

	if (timerPlaying == 1) {
		document.getElementById("yearField").style.display = "none";
		document.getElementById("monthField").style.display = "none";
		document.getElementById("dayField").style.display = "none";
		document.getElementById("hourField").style.display = "none";
		document.getElementById("minuteField").style.display = "none";
		document.getElementById("secondField").style.display = "none";

		updateClock();

		if (timer == null) {
			timer = setInterval("updateClock();", 1000);
		}
	} else {
		stopTimer();
	}
}

// FUNCTION stopTimer
// stop timer
//--------------------------------
function stopTimer() {
	if (timer != null) {
		clearInterval(timer);
		timer = null;
	}
}

// FUNCTION erasePrefs
// erase preferences when closed
//--------------------------------
function erasePrefs()
{
	if (window.widget) widget.setPreferenceForKey(null,locationKey);
}

// FUNCTION updateClock
// update clock and calculate LST
//--------------------------------
function updateClock() {

	// set longitude of selected location
	var location = document.getElementById("location").value;
	var locLong = 0.0;
	var count = locationsList.length;
	for (var i=0; i<count; i++) {
		if (location == locationsList[i].name) {
			locLong = locationsList[i].longitude;
			break;
		}
	}

	// update location name label
	document.getElementById("curLocation").innerText = location;

	// get current date and time zone
	var date = new Date();
	var tz = date.getTimezoneOffset();

	// make current date string YYYY/MM/DD hh:mm:ss
	//var year = date.getYear();
	//if (year < 1900) year += 1900;
	var year = date.getFullYear();
	var month = date.getMonth() + 1;
	if (month < 10) month = "0" + month;
	var day = date.getDate();
	if (day < 10) day = "0" + day;
	var hours = date.getHours();
	if (hours < 10) hours = "0" + hours;
	var minutes = date.getMinutes();
	if (minutes < 10) minutes = "0" + minutes;
	var seconds = date.getSeconds();
	if (seconds < 10) seconds = "0" + seconds;
//	 var time = year + "/" + month + "/" + day + " " + hours + ":" + minutes+":"+seconds;

	// copy date string to fields
	document.getElementById("yearField").value = year;
	document.getElementById("monthField").value = month;
	document.getElementById("dayField").value = day;
	document.getElementById("hourField").value = hours;
	document.getElementById("minuteField").value = minutes;
	document.getElementById("secondField").value = seconds;

	calcLST();
}

function calcLST() {

	// set longitude of selected location
	var location = document.getElementById("location").value;
	var locLong = 0.0;
	var count = locationsList.length;
	for (var i=0; i<count; i++) {
		if (location == locationsList[i].name) {
			locLong = locationsList[i].longitude;
			break;
		}
	}

	// update location name label
	document.getElementById("curLocation").innerText = location;

	// get current date and time zone
	var date = new Date();
	var tz = date.getTimezoneOffset();

	// copy date string from fields
	var year = document.getElementById("yearField").value;
	var month = document.getElementById("monthField").value;
	var day = document.getElementById("dayField").value;
	var hours = document.getElementById("hourField").value;
	var minutes = document.getElementById("minuteField").value;
	var seconds = document.getElementById("secondField").value;

	var time = year + "-" + month + "-" + day + " " + hours + ":" + minutes+":"+seconds;

	date.setFullYear(parseInt(year,10));
	date.setMonth(parseInt(month,10)-1);
	date.setDate(parseInt(day,10));
	date.setHours(parseInt(hours,10));
	date.setMinutes(parseInt(minutes,10));
	date.setSeconds(parseInt(seconds,10));

	// calculate LST and make string hh:mm:ss
	date2000 = new Date(2000,0,1,12,0);
	t2000 = ((date.getTime() - date2000.getTime())/1000 + tz*60)/86400/36525;
	lst = 6.697374558 + t2000 * (2400.05133691 + t2000 * (2.586222e-5 - t2000*1.72222e-9 ) ) +
		parseFloat(hours) + parseFloat(minutes)/60 + parseFloat(seconds)/3600 + locLong/15 + tz/60;
	lst -= 24*Math.floor(lst/24);
	lstHours = Math.floor(lst);
	if (lstHours < 10) lstHours = "0" + lstHours
	lst = (lst - lstHours)*60;
	lstMinutes = Math.floor(lst);
	if (lstMinutes < 10) lstMinutes = "0" + lstMinutes
	lst = (lst - lstMinutes)*60;
	lstSeconds = Math.floor(lst);
	if (lstSeconds < 10) lstSeconds = "0" + lstSeconds;
	lst = lstHours + ":" + lstMinutes + ":" + lstSeconds;

	// update clock and LST strings
	document.getElementById("clock").innerText = time;
	document.getElementById("LST").innerText = lst;
}

// FUNCTION showBackside
// function to flip the widget: front->back
//--------------------------------
function showBackside(event) {
	var front = document.getElementById("front");
	var back = document.getElementById("back");

	if (window.widget) widget.prepareForTransition("ToBack");

	front.style.display = "none";
	back.style.display = "block";

	if (window.widget) {
		setTimeout("widget.performTransition();", 0);

		// set location name and longitude fields
		var value = document.getElementById("location").value;
		for(var i=0; i<5; i++) {
			if (value == locationsList[i].name) {
				document.getElementById("locNameField").value = locationsList[i].name;
				document.getElementById("locLongField").value = locationsList[i].longitude;
				break;
			}
		}
	}

	// hide info button
	var infoRollie = document.getElementById("infoRollie");
	infoRollie.style.opacity = "0.0";
}

// FUNCTION showFrontside
// function to flip the widget: back->front
//--------------------------------
function showFrontside(event) {
	var front = document.getElementById("front");
	var back = document.getElementById("back");

	if (window.widget) widget.prepareForTransition("ToFront");

	front.style.display = "block";
	back.style.display = "none";

	if (window.widget) setTimeout("widget.performTransition();", 0);

	if (timerPlaying == 0) {
		showField(yearField);
		yearField.focus();
		calcLST();
	}
}

// FUNCTION popupChanged
// function called when location popup is changed
//--------------------------------
function popupChanged() {
	var value = document.getElementById("location").value;
	if (window.widget) {
		// save selected location name to prefs
		widget.setPreferenceForKey(value,locationKey);
		for(i=0; i<5; i++) {
			if (value == locationsList[i].name) j = i;
		}
		// update name and longitude fields
		document.getElementById("locNameField").value = locationsList[j].name;
		document.getElementById("locLongField").value = locationsList[j].longitude;
	}
}

// FUNCTION updateLocationSelect
// function to update location popup menu
//--------------------------------
function updateLocationSelect()
{
	var select = document.getElementById ('location');

	// remove all children
	while (select.hasChildNodes()) select.removeChild(select.firstChild);

	var c = locationsList.length;

	for (var i = 0; i < c; ++i) {
		var element = document.createElement("option");
		element.innerText = locationsList[i].name;
		select.appendChild (element);
	}
}

// FUNCTION changeLocationPopup
// function called when "change" button is clicked
//--------------------------------
function changeLocationPopup()
{
	var value = document.getElementById("location").value;
	if (window.widget) {
		for(var i=0; i<5; i++) {
			if (value == locationsList[i].name) {
				locationsList[i].name = document.getElementById("locNameField").value;
				locationsList[i].longitude = document.getElementById("locLongField").value;
				widget.setPreferenceForKey(locationsList[i].name,"locListName" + i);
				widget.setPreferenceForKey(locationsList[i].longitude,"locListLong" + i);
				widget.setPreferenceForKey(locationsList[i].name,locationKey);
				break;
			}
		}
		updateLocationSelect();
		document.getElementById("location").value = locationsList[i].name;
	}
}

// FUNCTION playIconClicked
// function called when play/stop icon is clicked
//--------------------------------
function playIconClicked()
{
	if (timerPlaying == 1) {
		timerPlaying = 0;
		document.getElementById("yearField").style.display = "block";
		document.getElementById("monthField").style.display = "block";
		document.getElementById("dayField").style.display = "block";
		document.getElementById("hourField").style.display = "block";
		document.getElementById("minuteField").style.display = "block";
		document.getElementById("secondField").style.display = "block";
		stopTimer();
		document.getElementById("yearField").focus();
		timerStatus.src = "timerStopOver.png";
	} else {
		timerPlaying = 1;
//		document.getElementById("yearField").style.display = "none";
//		document.getElementById("monthField").style.display = "none";
//		document.getElementById("dayField").style.display = "none";
//		document.getElementById("hourField").style.display = "none";
//		document.getElementById("minuteField").style.display = "none";
//		document.getElementById("secondField").style.display = "none";
		startTimer();
		timerStatus.src = "timerPlayOver.png";
	}
}

function playIconOver()
{
	if (timerPlaying == 1) {
		timerStatus.src = "timerPlayOver.png";
	} else {
		timerStatus.src = "timerStopOver.png";
	}
}
function playIconOut()
{
	if (timerPlaying == 1) {
		timerStatus.src = "timerPlay.png";
	} else {
		timerStatus.src = "timerStop.png";
	}
}

// FUNCTION fieldChanged
// function called when year/month/day/hour/minute/second changed
//--------------------------------
function fieldChanged(fieldId,minVal,maxVal)
{
	fieldId.style.opacity = "1.0";
	fieldId.value = formInt(fieldId.value,minVal,maxVal);
	calcLST();
}

// FUNCTION formInt
//--------------------------------
function formInt(inStr,minVal,maxVal)
{
	var val = parseInt(inStr,10);
	if (isNaN(val)) val = 0;
	val = Math.min(maxVal,Math.max(val,minVal));
	if (val < 10) val = "0"+val;
	return val;
}

// FUNCTION showField
//--------------------------------
function showField(fieldId)
{
	if (timerPlaying == 0) {
		fieldId.style.opacity = "1.0";
	}
}

// FUNCTION hideField
//--------------------------------
function hideField(fieldId)
{
	fieldId.style.opacity = "0.0";
}
